import React, {
  useState,
  useCallback,
  useEffect,
} from 'react';

import ReactDOM from 'react-dom';
import {
  csv,
  arc,
  pie,
  scaleBand,
  scaleLinear,
  max,
} from 'd3';

const width = 950;
const height = 480;
const margin = {
  top: 20,
  right: 80,
  left: 180,
  bottom: 50,
};

const csvUrl =
  'https://gist.githubusercontent.com/curran/0ac4077c7fc6390f5dd33bf5c06cb5ff/raw/605c54080c7a93a417a3cea93fd52e7550e76500/UN_Population_2019.csv';

const flagData = [
  {
    County: 'China',
    URL:
      'https://raw.githubusercontent.com/joielechong/iso-country-flags-svg-collection/master/svg/country-4x3/cn.svg',
  },
  {
    County: 'India',
    URL:
      'https://raw.githubusercontent.com/joielechong/iso-country-flags-svg-collection/master/svg/country-4x3/in.svg',
  },
  {
    County: 'United States of America',
    URL:
      'https://raw.githubusercontent.com/joielechong/iso-country-flags-svg-collection/master/svg/country-4x3/us.svg',
  },

  {
    County: 'Brazil',
    URL:
      'https://raw.githubusercontent.com/joielechong/iso-country-flags-svg-collection/master/svg/country-4x3/br.svg',
  },
  {
    County: 'Europe',
    URL:
      'https://raw.githubusercontent.com/joielechong/iso-country-flags-svg-collection/master/svg/country-4x3/bd.svg',
  },
  {
    County: 'Canada',
    URL:
      'https://raw.githubusercontent.com/joielechong/iso-country-flags-svg-collection/master/svg/country-4x3/mx.svg',
  },
];

const App = () => {
  const [data, setData] = useState(null);

  useEffect(() => {
    // useEffect stops it looping every time the data is loaded because that will change the state.
    const row = (d) => {
      d.Population = +d['2020'];
      return d;
    };
    csv(csvUrl, row).then((data) => {
      setData(data.slice(0, 10));
    });
  }, []);

  if (!data) {
    return <pre>Loading ... </pre>;
  }

  // This will only execute once the data has loaded...

  console.log(data[0]);
  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  const svgURL = function (d) {
    const URLResult = flagData.find(
      ({ County }) => County === d
    );
    if (!URLResult) {
      return;
    }
    return URLResult.URL;
  };

  const yScale = scaleBand()
    .domain(data.map((d) => d.Country))
    .range([0, innerHeight])
    .paddingInner(0.1);

  const xScale = scaleLinear()
    .domain([0, max(data, (d) => d.Population)])
    .range([0, innerWidth])
    ;

  // console.log(xScale.ticks(10)); // This defaults to 10
  // console.log(yScale.domain());

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left},${margin.top})`}
      >
        {xScale.ticks().map((tickValue) => (
          <g
            transform={`translate(${xScale(tickValue)},0)`}
          >
            <line
              //  x1={0}  y1={0}  x2={0}
              y2={innerHeight}
              stroke="black"
            />
            <text
              y={innerHeight + 3}
              dy="0.71em"
              text-anchor="middle"
            >
              {tickValue}
            </text>
          </g>
        ))}

        {yScale.domain().map((tickValue) => (
          <text
            y={0}
            dx="0.32em"
            text-anchor="end"
            x={-10}
            y={(yScale(tickValue) + yScale.bandwidth() / 2)+3}
          >
            {tickValue}
          </text>
        ))}

        {data.map((d) => (
          <rect
            x={0}
            y={yScale(d.Country)}
            height={yScale.bandwidth()}
            width={xScale(d.Population)}
          />
        ))}

        {data.map((d) => (
          // Here i've added the code to add the image and rect (which gives the flags a boarder)
          // Ideally the x, y, width and height should probably be in the group element.
          // x,y you can use translate below but getting the scale to work was a lot more difficult!
					
          <g>
            <image
              x={xScale(d.Population) + 5}
              y={yScale(d.Country) + 5}
              height={yScale.bandwidth() - 10}
              width={(yScale.bandwidth() - 10) / 0.75} // This uses the height to detimin with width as a factor
              xlink:href={svgURL(d.Country)} // Originaly tried to add the function here but it wouldn't enter it for some reason but no errors were logged
            />

            <rect
              x={xScale(d.Population) + 5}
              y={yScale(d.Country) + 5}
              height={yScale.bandwidth() - 10}
              width={(yScale.bandwidth() - 10) / 0.75}
              fill="none"
              stroke="grey"
              stroke-width="0.5"
            />
          </g>
  
        ))}
        
        
        
      </g>
    </svg>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);



/*  >> Tried to use scale but didn't work as expectd

        <g transform={`translate(${xScale(d.Population) + 5},${yScale(d.Country) + 5})  scale(0.06)`}>
            
            <image
	          	height="100%"
              width="100%"
              xlink:href={svgURL(d.Country)} // Originaly tried to add the function here but it wouldn't enter it for some reason but no errors were logged
            />

            <rect
	          	height="100%"
              width="100%"
              fill="none"
              stroke="grey"
              stroke-width="0.5"
            />
          </g>
          
          
          */